import logging
import os
from dataclasses import dataclass


@dataclass
class CredentialClass:
    _name: str
    _credentials_inited: bool = False

    def __init__(self, **kwargs):
        if self._credentials_inited:
            return
        self._credentials_inited = True

        cls_keys = [
            i
            for i in self.__dict__.keys()
            if not i.startswith("__") and not i.startswith("_")
        ]

        for key in cls_keys:
            env_var = f"{self._name.upper()}_{key.upper()}"
            value = os.getenv(env_var, None)
            if value:
                setattr(self, key, value)
            else:
                logging.debug(
                    f"didn't got {env_var} variable, using standard({getattr(self, key)}) one!"
                )

        for key, value in kwargs.items():
            setattr(self, key, value)


class PostgreSqlCredentials(CredentialClass):
    _name = "postgres"
    host: str = "localhost"
    port: str = "5432"
    password: str = "postgres"
    user: str = "postgres"
    db_name: str = "postgres"

    @property
    def dsn(self):
        return (
            f"postgresql://{self.user}:{self.password}@"
            f"{self.host}:{self.port}/{self.db_name}"
        )

    @property
    def test_db_name(self):
        return f"{self.db_name}_test"



import contextlib
from copy import deepcopy
from dataclasses import replace
from os import chdir, getenv
from os.path import abspath, dirname
from typing import Optional

from .data import PostgreSqlCredentials


class Config:
    def __init__(self, abs_path=__file__):
        self.abs_path = abs_path
        self.try_to_load_dot_env(self.abs_path)
        self.SERVICE_NAME = self.get_service_name(self.abs_path)

        self.postgres_creds: Optional[PostgreSqlCredentials] = None

        self.LOG_LEVEL = getenv("LOG_LEVEL", "INFO")

    @property
    def postgres(self) -> PostgreSqlCredentials:
        if not self.postgres_creds:
            raise AttributeError("Init credential first!")

        return self.postgres_creds

    @postgres.setter
    def postgres(self, value):
        if type(value) != PostgreSqlCredentials:
            raise AttributeError("Init credential first!")
        self.postgres_creds = value

    def init_postgres(self) -> None:
        self.postgres_creds = PostgreSqlCredentials()

    @staticmethod
    def try_to_load_dot_env(abs_path):
        try:
            # noinspection PyUnresolvedReferences
            from dotenv import load_dotenv

            abs_path = abspath(abs_path)
            dir_name = dirname(dirname(abs_path))
            chdir(dir_name)
            return load_dotenv(".env")
        except ImportError:
            pass

    @staticmethod
    def get_service_name(abs_path=__file__):
        abs_path = abspath(abs_path)
        dir_name = dirname(dirname(abs_path))
        return dir_name.split("/")[-1]

    @contextlib.contextmanager
    def temp_variable(self, key, value):
        split_keys = key.split(".")
        if len(split_keys) > 1:
            cached_class = getattr(self, split_keys[0])
            d = {split_keys[1]: value}
            new_dataclass = replace(cached_class, **d)
            setattr(self, split_keys[0], new_dataclass)
            yield self
            self.__setattr__(split_keys[0], cached_class)

        else:
            _class = deepcopy(self)
            cached_variable = getattr(_class, key)
            setattr(self, key, value)
            yield self
            self.__setattr__(key, cached_variable)



from setuptools import setup
from kz159_utils import version

setup(
    name="kz159_utils",
    version=version,
    packages=["kz159_utils"],
    url="https://github.com/kz159/simple_config",
    license="MIT",
    author="kz159",
    author_email="me@kz159.ru",
    description="",
)
